#ifndef __JZ_P32MX_CORE_H__
#define __JZ_P32MX_CORE_H__
#include <stdint.h>

static inline unsigned long read_gp() {
	unsigned int c = 0;
	__asm__ __volatile__("  \
		.set noreorder; \
		move    %0,$gp; \
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_gp(unsigned long c) {
	__asm__ __volatile__("\
		.set noreorder;         \
		move    $gp,%0;         \
		.set reorder" :: "r" (c));
}

static inline void write_t9(unsigned long c) {
	__asm__ __volatile__("\
		.set noreorder;         \
		move    $t9,%0;         \
		.set reorder" :: "r" (c));
}

static inline unsigned long read_sp() {
	unsigned int c = 0;
	__asm__ __volatile__("  \
		.set noreorder; \
		move    %0,$sp; \
		.set reorder" : "=r" (c): );
	return c;
}

static inline unsigned long read_c0_status() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;	\
		mfc0	%0,$12;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_status(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$12;		\
	.set reorder" :: "r" (c));
}

#define C0_STATUS_CU3		(1<<31)
#define C0_STATUS_CU2		(1<<30)
#define C0_STATUS_CU1		(1<<29)
#define C0_STATUS_CU0		(1<<28)
#define C0_STATUS_RP		(1<<27)
#define C0_STATUS_BEV		(1<<22)
#define C0_STATUS_SR		(1<<20)
#define C0_STATUS_NMI		(1<<19)

#define C0_STATUS_IM7		(1<<15)
#define C0_STATUS_IM6		(1<<14)
#define C0_STATUS_IM5		(1<<13)
#define C0_STATUS_IM4		(1<<12)
#define C0_STATUS_IM3		(1<<11)
#define C0_STATUS_IM2		(1<<10)
#define C0_STATUS_IM1		(1<<9)
#define C0_STATUS_IM0		(1<<8)

#define C0_STATUS_UM		(1<<4)
#define C0_STATUS_ERL		(1<<2)
#define C0_STATUS_EXL		(1<<1)
#define C0_STATUS_IE		(1<<0)

static inline unsigned long read_c0_cause() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;	\
		mfc0	%0,$13;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_cause(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$13;		\
	.set reorder" :: "r" (c));
}

#define C0_CAUSE_BD		(1<<31)
#define C0_CAUSE_CE_BIT		(28)
#define C0_CAUSE_CE_MASK	(3<<28)
#define C0_CAUSE_IV		(1<<23)
#define C0_CAUSE_WP		(1<<22)

#define C0_CAUSE_IP7		(1<<15)
#define C0_CAUSE_IP6		(1<<14)
#define C0_CAUSE_IP5		(1<<13)
#define C0_CAUSE_IP4		(1<<12)
#define C0_CAUSE_IP3		(1<<11)
#define C0_CAUSE_IP2		(1<<10)
#define C0_CAUSE_IP1		(1<<9)
#define C0_CAUSE_IP0		(1<<8)

#define C0_EXC_BIT		(2)
#define C0_EXC_MASK		(0x3f<<2)
#define C0_EXC_INT		(0<<2)
#define C0_EXC_MOD		(1<<2)
#define C0_EXC_TLBL		(2<<2)
#define C0_EXC_TLBS		(3<<2)
#define C0_EXC_ADEL		(4<<2)
#define C0_EXC_ADES		(5<<2)
#define C0_EXC_IBE		(6<<2)
#define C0_EXC_DBE		(7<<2)
#define C0_EXC_SYS		(8<<2)
#define C0_EXC_BP		(9<<2)
#define C0_EXC_RI		(10<<2)
#define C0_EXC_CPU		(11<<2)
#define C0_EXC_OV		(12<<2)
#define C0_EXC_TR		(13<<2)
#define C0_EXC_WATCH		(23<<2)
#define C0_EXC_MCHECK		(24<<2)

static inline unsigned long read_c0_epc() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;	\
		mfc0	%0,$14;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_epc(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$14;		\
	.set reorder" :: "r" (c));
}

static inline unsigned long read_c0_prid() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;	\
		mfc0	%0,$15;	\
		.set reorder" : "=r" (c): );
	return c;
}

#define C0_PRID_COMPANY_BIT		(16)
#define C0_PRID_COMPANY_MASK		(0xff<<16)
#define C0_PRID_COMPANY_MIPS		(1<<16)
#define C0_PRID_PROCESSOR_BIT		(8)
#define C0_PRID_PROCESSOR_MASK		(0xff<<8)
#define C0_PRID_PROCESSOR_4KC		(0x80<<8)
#define C0_PRID_PROCESSOR_4KM		(0x83<<8)
#define C0_PRID_REVISION_BIT		(0)
#define C0_PRID_REVISION_MASK		(0xff<<0)

static inline unsigned long read_c0_errorepc() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;	\
		mfc0	%0,$30;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_errorepc(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$30;		\
	.set reorder" :: "r" (c));
}

extern void irq_enable();
static inline void __irq_enable()
{
	unsigned long c;
	write_c0_cause(0);
	c = read_c0_status();
	c &= 0x1f;
	c |= 0x01;
	write_c0_status(c);
}

extern unsigned long irq_disable();
static inline unsigned long __irq_disable()
{
	unsigned long c;
#if defined(__32M4KCORE__)
	__asm__ __volatile__("	\
		.set noreorder;	\
		di	%0;	\
		.set reorder" : "=r" (c): );
#else
	c = read_c0_status();
	write_c0_status(c & ~0x1f);
#endif
	return c;
}

extern void irq_restore(unsigned long c);
static inline void __irq_restore(unsigned long c)
{
#if defined(__32M4KCORE__) && defined(USE_MIPS32R2_EI)
	if (! (c & C0_STATUS_IE)) {
		__asm__ __volatile__("\
			.set noreorder;		\
			ei	      ;		\
			.set reorder" );
	}
#else
	write_c0_status(c);
#endif
}

extern void nop();
static inline void __nop()
{
	__asm__ __volatile__("\
		.set noreorder;		\
		nop	      ;		\
		.set reorder" );
}

extern void cli();
static inline void __cli()
{
	__asm__ __volatile__("\
		.set noreorder;		\
		di	      ;		\
		.set reorder" );
}

extern void sei();
static inline void __sei()
{
	__asm__ __volatile__("\
		.set noreorder;		\
		ei	      ;		\
		.set reorder" );
}

static inline void __delay_loop_1(unsigned int loops)
{
        __asm__ __volatile__ (
        "       .set    noreorder    \n"
        "       .align  3            \n"
        "1:     bnez    %0, 1b       \n"
        "       subu    %0, 1        \n"
        "       .set    reorder      \n"
        : "=r" (loops)
        : "0" (loops));
}

extern void delay_us(unsigned int us);
extern void delay_ms(unsigned int ms);

#if defined(__32M4KCORE__)
static inline int __fls(uint32_t x)
{
	/* Count Leading Zeros (from MSB) */
	__asm__("clz %0, %1" : "=r" (x) : "r" (x));
	return 32 - x;
}

static inline unsigned long read_c0_intctl() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$12,1;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_intctl(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$12,1;	\
	.set reorder" :: "r" (c));
}

#define C0_INTCTL_PF_BIT		(22)
#define C0_INTCTL_PF_MASK		(0x01<<22)
#define C0_INTCTL_ICE_BIT		(21)
#define C0_INTCTL_ICE_MASK		(0x01<<21)
#define C0_INTCTL_STKDEC_BIT		(16)
#define C0_INTCTL_STKDEC_MASK		(0x1f<<16)
#define C0_INTCTL_CLREXL_BIT		(15)
#define C0_INTCTL_CLREXL_MASK		(0x01<<15)
#define C0_INTCTL_APE_BIT		(14)
#define C0_INTCTL_APE_MASK		(0x01<<14)
#define C0_INTCTL_USESTK_BIT		(13)
#define C0_INTCTL_USESTK_MASK		(0x01<<13)
#define C0_INTCTL_VS_BIT		(5)
#define C0_INTCTL_VS_MASK		(0x1f<<5)

static inline unsigned long read_c0_srsctl() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$12,2;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_srsctl(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$12,2;	\
	.set reorder" :: "r" (c));
}

#define C0_SRSCTL_HSS_BIT		(26)
#define C0_SRSCTL_HSS_MASK		(0x0f<<26)
#define C0_SRSCTL_EICSS_BIT		(18)
#define C0_SRSCTL_EICSS_MASK		(0x0f<<18)
#define C0_SRSCTL_ESS_BIT		(12)
#define C0_SRSCTL_ESS_MASK		(0x0f<<12)
#define C0_SRSCTL_PSS_BIT		(6)
#define C0_SRSCTL_PSS_MASK		(0x0f<<6)
#define C0_SRSCTL_CSS_BIT		(0)
#define C0_SRSCTL_CSS_MASK		(0x0f<<0)

static inline unsigned long read_c0_srsmap() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$12,3;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_srsmap(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$12,3;	\
	.set reorder" :: "r" (c));
}

#define C0_SRSMAP_SSV7_BIT		(28)
#define C0_SRSMAP_SSV7_MASK		(0x0f<<28)
#define C0_SRSMAP_SSV6_BIT		(24)
#define C0_SRSMAP_SSV6_MASK		(0x0f<<24)
#define C0_SRSMAP_SSV5_BIT		(20)
#define C0_SRSMAP_SSV5_MASK		(0x0f<<20)
#define C0_SRSMAP_SSV4_BIT		(16)
#define C0_SRSMAP_SSV4_MASK		(0x0f<<16)
#define C0_SRSMAP_SSV3_BIT		(12)
#define C0_SRSMAP_SSV3_MASK		(0x0f<<12)
#define C0_SRSMAP_SSV2_BIT		(8)
#define C0_SRSMAP_SSV2_MASK		(0x0f<<8)
#define C0_SRSMAP_SSV1_BIT		(4)
#define C0_SRSMAP_SSV1_MASK		(0x0f<<4)
#define C0_SRSMAP_SSV0_BIT		(0)
#define C0_SRSMAP_SSV0_MASK		(0x0f<<0)

static inline unsigned long read_c0_ebase() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$15,1;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_ebase(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$15,1;	\
	.set reorder" :: "r" (c));
}

static inline unsigned long read_c0_count() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$9;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_count(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$9;	\
	.set reorder" :: "r" (c));
}

static inline unsigned long read_c0_compare() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$11;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_compare(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$11;	\
	.set reorder" :: "r" (c));
}

static inline unsigned long read_c0_config() {
	unsigned int c = 0;
	__asm__ __volatile__("	\
		.set noreorder;		\
		mfc0	%0,$16;	\
		.set reorder" : "=r" (c): );
	return c;
}

static inline void write_c0_config(unsigned long c) {
	__asm__ __volatile__("\
	.set noreorder;		\
	mtc0	%0,$16;	\
	.set reorder" :: "r" (c));
}
#endif /* __32M4KCORE__ */
#endif /* __JZ_P32MX_CORE_H__ */
