#ifndef _JZ_P32MX_INTC_H_
#define _JZ_P32MX_INTC_H_
#include <stdint.h>
  #define INTCON           (*(volatile unsigned *)0xBF881000)
  #define INTCON_CLR       (*(volatile unsigned *)0xBF881004)
  #define INTCON_SET       (*(volatile unsigned *)0xBF881008)
  #define INTCON_INV       (*(volatile unsigned *)0xBF88100C)
  #define INTSTAT          (*(volatile unsigned *)0xBF881010)
  #define IPTMR            (*(volatile unsigned *)0xBF881020)
  #define IPTMR_CLR        (*(volatile unsigned *)0xBF881024)
  #define IPTMR_SET        (*(volatile unsigned *)0xBF881028)
  #define IPTMR_INV        (*(volatile unsigned *)0xBF88102C)

  /* x=0: IFS0 .. x=10: IFS1 */
  #define IFS(x)           (*(volatile unsigned *)(0xBF881030  + (x)*0x10))
  #define IFS_CLR(x)       (*(volatile unsigned *)(0xBF881034  + (x)*0x10))
  #define IFS_SET(x)       (*(volatile unsigned *)(0xBF881038  + (x)*0x10))
  #define IFS_INV(x)       (*(volatile unsigned *)(0xBF88103C  + (x)*0x10))

  /* x=0: IEC0 .. x=10: IEC1 */
  #define IEC(x)           (*(volatile unsigned *)(0xBF881060  + (x)*0x10))
  #define IEC_CLR(x)       (*(volatile unsigned *)(0xBF881064  + (x)*0x10))
  #define IEC_SET(x)       (*(volatile unsigned *)(0xBF881068  + (x)*0x10))
  #define IEC_INV(x)       (*(volatile unsigned *)(0xBF88106C  + (x)*0x10))

  /* x=0: IPC0 .. x=10: IPC10 */
  #define IPC(x)           (*(volatile unsigned *)(0xBF881090 + (x)*0x10))
  #define IPC_CLR(x)       (*(volatile unsigned *)(0xBF881094 + (x)*0x10))
  #define IPC_SET(x)       (*(volatile unsigned *)(0xBF881098 + (x)*0x10))
  #define IPC_INV(x)       (*(volatile unsigned *)(0xBF88109C + (x)*0x10))

    #define INTCON_INT0EP                  0x00
    #define INTCON_INT1EP                  0x01
    #define INTCON_INT2EP                  0x02
    #define INTCON_INT3EP                  0x03
    #define INTCON_INT4EP                  0x04
    #define INTCON_TPC                     0x08
    #define INTCON_MVEC                    0x0C
    #define INTCON_SS0                     0x10
    #define INTSTAT_VEC                    0x00
    #define INTSTAT_SRIPL                  0x08

  /* IRQ definition for 1xx/2xx */
  #if defined(__32MX1XX__) || defined(__32MX2XX__)
    #define IRQ_CT                    0
    #define IRQ_CS0                   1
    #define IRQ_CS1                   2
    #define IRQ_INT0                  3
    #define IRQ_T1                    4
    #define IRQ_IC1E                  5
    #define IRQ_IC1                   6
    #define IRQ_OC1                   7
    #define IRQ_INT1                  8
    #define IRQ_T2                    9
    #define IRQ_IC2E                  10
    #define IRQ_IC2                   11
    #define IRQ_OC2                   12
    #define IRQ_INT2                  13
    #define IRQ_T3                    14
    #define IRQ_IC3E                  15
    #define IRQ_IC3                   16
    #define IRQ_OC3                   17
    #define IRQ_INT3                  18
    #define IRQ_T4                    19
    #define IRQ_IC4E                  20
    #define IRQ_IC4                   21
    #define IRQ_OC4                   22
    #define IRQ_INT4                  23
    #define IRQ_T5                    24
    #define IRQ_IC5E                  25
    #define IRQ_IC5                   26
    #define IRQ_OC5                   27
    #define IRQ_AD1                   28
    #define IRQ_FSCM                  29
    #define IRQ_RTCC                  30
    #define IRQ_FCE                   31
    #define IRQ_CMP1                  32
    #define IRQ_CMP2                  33
    #define IRQ_CMP3                  34
    #define IRQ_USB                   35
    #define IRQ_SPI1E                 36
    #define IRQ_SPI1RX                37
    #define IRQ_SPI1TX                38
    #define IRQ_U1E                   39
    #define IRQ_U1RX                  40
    #define IRQ_U1TX                  41
    #define IRQ_I2C1B                 42
    #define IRQ_I2C1S                 43
    #define IRQ_I2C1M                 44
    #define IRQ_CNA                   45
    #define IRQ_CNB                   46
    #define IRQ_CNC                   47
    #define IRQ_PMP                   48
    #define IRQ_PMPE                  49
    #define IRQ_SPI2E                 50
    #define IRQ_SPI2RX                51
    #define IRQ_SPI2TX                52
    #define IRQ_U2E                   53
    #define IRQ_U2RX                  54
    #define IRQ_U2TX                  55
    #define IRQ_I2C2B                 56
    #define IRQ_I2C2S                 57
    #define IRQ_I2C2M                 58
    #define IRQ_CTMU                  59
    #define IRQ_DMA0                  60
    #define IRQ_DMA1                  61
    #define IRQ_DMA2                  62
    #define IRQ_DMA3                  63

    #define __intc_mask_irq(v) {			\
        if ((v) & 0x20)  IEC_CLR(1) = (1 << ((v) & 0x1f)); \
	else             IEC_CLR(0) = (1 << ((v) & 0x1f)); \
    }
    #define __intc_unmask_irq(v) {			\
        if ((v) & 0x20)  IEC_SET(1) = (1 << ((v) & 0x1f)); \
	else             IEC_SET(0) = (1 << ((v) & 0x1f)); \
    }
    #define __intc_ack_irq(v) {				\
        if ((v) & 0x20)  IFS_CLR(1) = (1 << ((v) & 0x1f)); \
	else             IFS_CLR(0) = (1 << ((v) & 0x1f)); \
    }
  #endif

#endif /* JZ_P32MX_INTC_H_ */
