#include <p32mx/p32mx_core.h>
#include <p32mx/interrupt.h>
#include <stdint.h>
#include <string.h>

extern int (* __vectors[64])(void);

void interrupt_enable()
{
	uint32_t c0_status;

#if defined(__32M4KCORE__) && defined(USE_SRS)
	uint32_t srsctl;

	INTCON_CLR = (1 << INTCON_MVEC);
	INTCON_SET = (1 << INTCON_SS0);
	srsctl = 0x01 << C0_SRSCTL_EICSS_BIT;
	write_c0_srsctl(srsctl);
#endif
	c0_status = read_c0_status();
	c0_status &= 0xffa70000;	/* BEV=0 , SR=0,  NMI=0 */
					/* UM=0 , ERL=0 , EXL=0, IE=0 */
	c0_status |= 0x0401;		/* IPL=2 , IE = 1 */
	write_c0_status(c0_status);

	/* unmask all first */
	IEC_SET(0) = 0xffffffff;
	IEC_SET(1) = 0xffffffff;
}

#if defined(__32M4KCORE__)
#define ffs(x)	__fls(x)
#endif

uint32_t intent(uint32_t c0_status, uint32_t c0_cause, uint32_t c0_epc)
{
	uint32_t isr;
	int v,r;

	c0_status &= ~0xe;
	write_c0_status(c0_status & ~0x11); /* kernel mode + irq_disable */
	do {
		isr = IFS(0);
		v = ffs(isr);
		if (v) {
			v--;
			__intc_mask_irq(v & 0x1f);
			r = (__vectors[v])();
			if (r) {
				__intc_unmask_irq(v & 0x1f);
			}
		}
	} while (isr);
	do {
		isr = IFS(1);
		v = ffs(isr);
		if (v) {
			v--;
			__intc_mask_irq((v & 0x1f) | 0x20);
			r = (__vectors[v | 0x20])();
			if (r) {
				__intc_unmask_irq((v & 0x1f) | 0x20);
			}
		}
	} while (isr);

	write_c0_cause(0);
	return c0_status;
}
