#include <p32mx/p32mx_intc.h>
#include <p32mx/p32mx_timer.h>
#include <p32mx/p32mx_clkc.h>
#include <p32mx/p32mx_core.h>
#include <p32mx/interrupt.h>
#include <p32mx/ostimer.h>

static volatile uint32_t current_cycle_high = 0;

union cycle_type {
	uint64_t cycle64;
	uint32_t cycle32[2];
};

uint32_t ostimer32()
{
	uint16_t hi,lo1,lo2;
	lo1 = TMR(4);
	hi = TMR(5);
	lo2 = TMR(4);
	if (lo1 < lo2) return ((uint32_t)hi << 16 ) | lo2;
	return ((uint32_t)TMR(5) << 16 ) | lo2;
}

uint64_t ostimer64()
{
	uint32_t hi,lo1,lo2;
	lo1 = ostimer32();
	hi = current_cycle_high;
	lo2  = ostimer32();
	if (lo1 < lo2) return ((uint64_t)hi << 32 ) | lo2;
	return ((uint64_t)current_cycle_high << 32 ) | lo2;
}

SIGNAL(IRQ_T5)
{
	current_cycle_high++;
	__intc_ack_irq(IRQ_T5);
	return IRQ_HANDLED;
}

int ostimer_clk()
{
	int div;
	extern unsigned __sysclk;
	delay_us(1);
	
	div = (T_CON(4) >> T_CON_TCKPS) & 0x3;
	if (div == 7) div++;
	div +=  ( OSCCON >> OSCCON_PBDIV ) & 0x3;
	return __sysclk >> div;
}

static inline void __ostimer_start(uint32_t prescale)
{
	__intc_mask_irq(IRQ_T5);
	current_cycle_high = 0;
	T_CON(5) = 0;				/* disable timer clock */
	T_CON(4) = prescale << T_CON_TCKPS;	/* disable timer clock */
	__intc_ack_irq(IRQ_T5);
	TMR(4) = 0;
	TMR(5) = 0;
	PR(4) = 0xffff;
	PR(5) = 0xffff;	/* free running */
	T_CON_SET(4) = (1 << T_CON_T32);/* 32bit mode */
	T_CON_SET(4) = (1 << T_CON_ON); /* start counting up */
	__intc_unmask_irq(IRQ_T5);
}

void ostimer_start(int is_fast)
{
	uint32_t prescale;
	if (!is_fast) {
		prescale = 6; // 1/64
	} else {
		prescale = 0; // 1/1
	}
	__ostimer_start(prescale);
}


void ostimer_stop()
{
	__intc_mask_irq(IRQ_T5);
	T_CON(5) = 0;				/* disable timer clock */
	T_CON(4) = 0;				/* disable timer clock */
	__intc_ack_irq(IRQ_T5);
}
